# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import models
from datetime import datetime, timedelta
from portailuser.models import Ecole, PortailUser
from gestion import fields


DUREES = (
    (30, "30 minutes"),
    (60, "1 heure"),
    (90, "1 heure 30"),
    (120, "2 heures"),
    (150, "2 heures 30"),
    (180, "3 heures"),
    (210, "3 heures 30"),
    (240, "4 heures"),
)


class Formation(models.Model):
    nom = models.CharField(verbose_name="Nom de la formation", max_length=32)
    description = models.TextField(
        verbose_name="Description",
        help_text="Penser à préciser le niveau prérequis, et éventuellement détailler le programme…",
    )
    date_heure = fields.DateTimeField(verbose_name="Date et heure")
    duree = models.IntegerField(verbose_name="Durée", choices=DUREES)
    responsable = models.ForeignKey(
        PortailUser,
        verbose_name="Responsable",
        null=True,
        help_text="Laissez vide pour utiliser votre nom",
        on_delete=models.CASCADE,
    )
    nb_fixes = models.PositiveSmallIntegerField(verbose_name="Nombre de places (machines fournies)", default=5)
    nb_portables = models.PositiveSmallIntegerField(verbose_name="Nombre de places (portables)", default=5)
    lieu = models.CharField(verbose_name="Lieu de rendez-vous", max_length=32)
    url_support = models.URLField(verbose_name="URL du support", blank=True)
    ecole = models.ForeignKey(Ecole, editable=False, on_delete=models.CASCADE)

    inscrits = models.ManyToManyField(
        PortailUser,
        verbose_name="Inscrits",
        related_name="+",
        through="Inscription",
        editable=False,
    )

    def __str__(self):
        return f"{self.nom} - {self.date_heure} - {self.responsable}"

    def get_url(self):
        return f"//{settings.URLS_SITES[self.ecole.id - 1]}"

    def est_inscrit(self, user):
        inscr = Inscription.objects.filter(formation=self, inscrit=user)
        return bool(inscr)

    def nb_places_total(self):
        return self.nb_fixes + self.nb_portables

    def occ_total(self):
        return Inscription.objects.filter(formation=self).count()

    def occ_fixes(self):
        return Inscription.objects.filter(formation=self, a_portable=False).count()

    def occ_portables(self):
        return Inscription.objects.filter(formation=self, a_portable=True).count()

    def places_fixes(self):
        return self.nb_fixes - self.occ_fixes()

    def places_portables(self):
        return self.nb_portables - self.occ_portables()

    def est_complete(self):
        return self.occ_fixes() >= self.nb_fixes and self.occ_portables() >= self.nb_portables

    def inscrire(self, user, a_portable):
        if Inscription.objects.filter(formation=self, inscrit=user).exists():
            return

        inscr = Inscription(formation=self, inscrit=user, a_portable=a_portable)
        inscr.save()

    def inscriptions(self):
        inscr = Inscription.objects.filter(formation=self)
        return ", ".join([i.inscrit.first_name + " " + i.inscrit.last_name for i in inscr])

    def desinscrire(self, user):
        """Pour se desinscrire"""
        desinscr = Inscription.objects.get(formation=self, inscrit=user)
        desinscr.delete()


class Inscription(models.Model):
    formation = models.ForeignKey(Formation, verbose_name="Formation", on_delete=models.CASCADE)
    inscrit = models.ForeignKey(PortailUser, verbose_name="Inscrit", on_delete=models.CASCADE)
    a_portable = models.BooleanField(verbose_name="A un portable", default=False)

    class Meta:
        unique_together = (("formation", "inscrit"),)


def prochaine_formation(user):
    maintenant = datetime.today()
    plus_tard = maintenant + timedelta(days=7)

    try:
        query_set = Inscription.objects.filter(
            inscrit=user,
            formation__date_heure__gt=maintenant,
            formation__date_heure__lt=plus_tard,
        )
        return query_set.order_by("formation__date_heure")[0].formation
    except IndexError:
        return None
